<?php

use App\libraries\Controller;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;

require MYROOT . '/vendor/autoload.php';
require MYROOT . '/vendor/phpmailer/src/Exception.php';
require MYROOT . '/vendor/phpmailer/src/PHPMailer.php';

class Users extends Controller
{
    public function __construct()
    {
        $this->userModel = $this->model('User');
    }

    public function login()
    {

        $data = [
            'title' => 'Login Page',
            'username' => '',
            'password' => '',
            'usernameError' => '',
            'passwordError' => '',

        ];

        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            $_POST = filter_input_array(INPUT_POST, FILTER_SANITIZE_STRING);

            $data = [
                'username' => trim($_POST['username']),
                'password' => trim($_POST['password']),
                'usernameError' => '',
                'passwordError' => '',

            ];

            if (empty($data['username'])) {
                $data['usernameError'] = "Enter username";
            }

            if (empty($data['password'])) {
                $data['passwordError'] = "Enter password";
            }

            if (empty($data['usernameError']) && empty($data['passwordError'])) {
                $loggedInUser = $this->userModel->login($data['username'], $data['password']);

                if ($loggedInUser) {
                    $this->createSession($loggedInUser);
                } else {
                    $data['passwordError'] = 'Password or username is incorrect.';

                    $this->view('users/login', $data);
                }
            }
        } else {
            $data = [
                'username' => '',
                'password' => '',
                'usernameError' => '',
                'passwordError' => '',

            ];
        }

        $this->view('users/login', $data);
    }

    public function createSession($user)
    {
        $_SESSION['id'] = $user->id;
        $_SESSION['username'] = $user->username;
        header('location:' . PATH . '/admin/index');
    }

    public function logout()
    {
        unset($_SESSION['id']);
        unset($_SESSION['username']);
        header('location:' . PATH . '/users/login');
    }

    public function register()
    {
        $data = [
            'username' => '',
            'email' => '',
            'password' => '',
            'confirmPassword' => '',
            'usernameError' => '',
            'emailError' => '',
            'passwordError' => '',
            'confirmPasswordError' => '',

        ];

        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            $_POST = filter_input_array(INPUT_POST, FILTER_SANITIZE_STRING);

            $data = [
                'username' => trim($_POST['username']),
                'email' => trim($_POST['email']),
                'password' => trim($_POST['password']),
                'confirm_password' => trim($_POST['confirm_password']),
                'usernameError' => '',
                'emailError' => '',
                'passwordError' => '',
                'confirmPasswordError' => '',

            ];

            $nameValidate = "/^[a-zA-Z0-9]*$/";
            $passwordValidate = "/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[$@$!%*?&])[A-Za-z\d$@$!%*?&]{8,}/";

            if (empty($data['username'])) {
                $data['usernameError'] = "Enter username";
            } elseif (!preg_match($nameValidate, $data['username'])) {
                $data['usernameError'] = 'Can only contain letters and numbers';
            }

            if (empty($data['email'])) {
                $data['emailError'] = "Enter your email";
            } elseif (!filter_var($data['email'], FILTER_VALIDATE_EMAIL)) {
                $data['emailError'] = 'Enter correct email';
            } else {
                if ($this->userModel->findUserByEmail($data['email'])) {
                    $data['emailError'] = 'Email is already taken';
                }
            }

            if (empty($data['password'])) {
                $data['passwordError'] = "Enter password";
            } elseif (strlen($data['password']) < 8) {
                $data['passwordError'] = 'Enter password with minimum 8 characters';
            } elseif (!preg_match($passwordValidate, $data['password'])) {
                $data['passwordError'] = "Enter password";
            }

            if (empty($data['confirm_password'])) {
                $data['confirmPasswordError'] = "Enter password";
            } else {
                if ($data['password'] != $data['confirm_password']) {
                    $data['confirmPasswordError'] = 'Password does not match';
                }
            }

            if (empty($data['usernameError']) && empty($data['emailError']) && empty($data['passwordError']) && empty($data['confirmPasswordError'])) {
                $data['password'] = password_hash($data['password'], PASSWORD_DEFAULT);
                $data['confirm_password'] = password_hash($data['confirm_password'], PASSWORD_DEFAULT);

                if ($this->userModel->register($data)) {
                    header('location: ' . PATH . '/users/login');
                }

                $data["notice"] = "You have registered successfully";
            }
        }

        $this->view('users/register', $data);
    }

    public function reset()
    {
        $data = [
            'email' => '',
            'emailError' => '',
            'success' => '',
        ];
        // reset password
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            $_POST = filter_input_array(INPUT_POST, FILTER_SANITIZE_STRING);

            $data = [
                'email' => trim($_POST['email']),
                'emailError' => '',
            ];

            if (empty($data['email'])) {
                $data['emailError'] = "Enter your email";
            }

            if (empty($data['emailError'])) {
                $email = $data['email'];

                try {

                    if ($this->userModel->findUserByEmail($email) == true) {
                        $expFormat = mktime(
                            date("H"),
                            date("i") + 70,
                            date("s"),
                            date("m"),
                            date("d"),
                            date("Y")
                        );
                        $expDate = date("Y-m-d H:i:s", $expFormat);
                        $key = md5($email);
                        $addKey = substr(md5(uniqid(rand(), 1)), 3, 10);
                        $key = $key . $addKey;
                        $this->userModel->insertToken($email, $key, $expDate);


                        $mail = new PHPMailer(true);

                        // Server settings
                        $mail->isSMTP();                                            //Send using SMTP
                        $mail->Host       = "mail.africabian.com";                     //Set the SMTP server to send through
                        $mail->SMTPAuth   = true;                                   //Enable SMTP authentication
                        $mail->Username   = "webmaster@africabian.com";                 //SMTP username
                        $mail->Password   = "HX,[=5Vt?l~l";                               //SMTP password
                        $mail->SMTPSecure = PHPMailer::ENCRYPTION_SMTPS;            //Enable implicit TLS encryption
                        $mail->Port       = "465";

                        $mail->setFrom("noreply@mail.com", "No Reply");
                        $mail->addAddress($email, "User");     // Add a recipient
                        $mail->IsHTML(true);
                        $subject = "Reset Password";
                        $message = '
                        <p>Click on the following link to reset your password</p>
                        ';
                        $message .= '<p><a href="' . PATH . '/users/newpassword?key=' . $key . '&email=' . $email . '&action=reset" target="_blank">
                        ' . PATH . '/users/newpassword
                        ?key=' . $key . '&email=' . $email . '&action=reset</a></p>';
                        $message .= '<p>-------------------------------------------------------------</p>';
                        $message .= '<p>Please be sure to copy the entire link into your browser.
                        The link will expire after 1 day for security reason.</p>';
                        $message .= '<p>If you did not request this forgotten password email, no action 
                        is needed, your password will not be reset. However, you may want to log into 
                        your account and change your security password as someone may have guessed it.</p>';
                        $mail->Subject = $subject;
                        $mail->Body = $message;
                        if ($mail->send()) {
                            $data['success'] = 'Email has been sent, find the link in your email.';
                        } else {
                            $data['success'] = 'Email has been sent, find the link in your email.';
                        }
                    } else {
                        $data['success'] = 'Email has been sent, find the link in your email.';
                    }
                } catch (Exception $e) {
                    echo "Error in sending email to client. Mailer Error: {$mail->ErrorInfo}";
                }
            }
        }
        $this->view('users/reset', $data);
    }

    public function newpassword()
    {

        if (isset($_POST["email"]) && isset($_POST["action"]) && ($_POST["action"] == "update")) {
            $_POST = filter_input_array(INPUT_POST, FILTER_SANITIZE_STRING);

            $errors = [
                'password' => trim($_POST['password']),
                'passwordConfirm' => trim($_POST['confirm_password']),
                'email' => trim($_POST['email']),
                'emailError' => '',
                'passwordError' => '',
                'passwordConfirmError' => '',
            ];

            $passwordValidate = "/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[$@$!%*?&])[A-Za-z\d$@$!%*?&]{8,}/";

            if (empty($errors['password'])) {
                $errors['passwordError'] = "Enter password";
            } elseif (strlen($errors['password']) < 8) {
                $errors['passwordError'] = 'Enter password with minimum 8 characters';
            } elseif (!preg_match($passwordValidate, $errors['password'])) {
                $errors['passwordError'] = "Password should contain atleast one uppercase, one lowercase, one digit and one special character";
            }

            if (empty($errors['passwordConfirm'])) {
                $errors['passwordConfirmError'] = "Enter Confirm password";
            }

            if ($errors['password'] != $errors['passwordConfirm']) {
                $errors['passwordConfirmError'] = "Passwords do not match";
            } else {
                if (empty($errors['passwordError']) && empty($errors['passwordConfirmationError'])) {
                    $errors['password'] = password_hash($errors['password'], PASSWORD_DEFAULT);
                    $errors['passwordConfirm'] = password_hash($errors['passwordConfirm'], PASSWORD_DEFAULT);

                    if ($this->userModel->updatePassword($errors)) {
                        $this->userModel->deleteKey($errors);
                        header('location: ' . PATH . '/users/login');
                    }
                }
            }

            $data = [
                'form' => '
                    <div class="container-fluid pt-4 login-page">
                        <div class="row justify-content-md-center pt-2 mt-4">
                            <div class="col-md-4 my-4 py-2 px-5 border bg-light shadow">
                                <form name="update" action="" method="POST">
                                    <div class="form-group">
                                        <label for="password" class="form-label">Password</label>
                                        <input type="password" name="password" class="form-control" id="password" aria-describedby="passwordHelp">
                                        <span class="text-danger">
                                            ' . $errors["passwordError"] . '
                                        </span>
                                        <input type="hidden" name="action" value="update" />
                                    </div>
                                    <div class="form-group">
                                        <label for="confirm_password" class="form-label">Confirm Password</label>
                                        <input type="password" name="confirm_password" class="form-control" id="confirm_password" aria-describedby="confirm_passwordHelp">
                                        <span class="text-danger">
                                            ' . $errors["passwordConfirmError"] . '
                                        </span>
                                        <input type="hidden" name="email" value="' . $errors["email"] . '"/>
                                    </div>
                                    <button type="submit" class="btn btn-secondary">Submit</button>
                                </form>
                                <br>

                            </div>
                        </div>
                    </div>
                    '
            ];

            $this->view('users/newpassword', $data);
        }

        if (
            isset($_GET["key"]) && isset($_GET["email"]) && isset($_GET["action"])
            && ($_GET["action"] == "reset") && !isset($_POST["action"])
        ) {
            $key = $_GET["key"];
            $email = $_GET["email"];
            $curDate = date("Y-m-d H:i:s");

            $results = $this->userModel->getKey($email, $key);

            $date = [
                "results" => $results,
            ];

            foreach ($date["results"] as $key) {
                $expDate = $key->expDate;
            }

            if ($expDate >= $curDate) {
                $data = [
                    'form' => '
                    <div class="container-fluid pt-4 login-page">
                        <div class="row justify-content-md-center pt-2 mt-4">
                            <div class="col-md-4 my-4 py-2 px-5 border bg-light shadow">
                                <form name="update" action="" method="POST">
                                    <div class="form-group">
                                        <label for="password" class="form-label">Password</label>
                                        <input type="password" name="password" class="form-control" id="password" aria-describedby="passwordHelp">
                                        <span class="text-danger">
                                            ' . $errors["passwordError"] . '
                                        </span>
                                        <input type="hidden" name="action" value="update" />
                                    </div>
                                    <div class="form-group">
                                        <label for="confirm_password" class="form-label">Confirm Password</label>
                                        <input type="password" name="confirm_password" class="form-control" id="confirm_password" aria-describedby="confirm_passwordHelp">
                                        <span class="text-danger">
                                            ' . $errors["passwordConfirmError"] . '
                                        </span>
                                        <input type="hidden" name="email" value="' . $email . '"/>
                                    </div>
                                    <button type="submit" class="btn btn-secondary">Submit</button>
                                </form>
                                <br>

                            </div>
                        </div>
                    </div>
                    '
                ];

                $this->view('users/newpassword', $data);
            } else {
                $data['form'] = 'The reset link is invalid or has expired.';
                $this->view('users/newpassword', $data);
            }
        }
    }
}
